import datetime
from .Exceptions import ParamError


def is_legal_array(array):
    if not array or not isinstance(array, list):
        return False
    elif len(array) <= 0 or not isinstance(array[0], float):
        return False
    return True


def is_legal_arrays(array):
    if not array or not isinstance(array, list):
        return False
    return True


def int_or_str(item):
    if isinstance(item, int):
        return str(item)
    else:
        return item


def is_correct_date_str(param):
    try:
        datetime.datetime.strptime(param, '%Y-%m-%d')
    except ValueError:
        raise ParamError('Incorrect data format, should be YYYY-MM-DD')
    return True


def legal_dimension(dim):
    if not isinstance(dim, int) or dim <= 0 or dim > 16384:
        return False
    return True


def parser_range_date(date):
    if isinstance(date, datetime.date):
        return date.strftime('%Y-%m-%d')
    elif isinstance(date, str):
        if not is_correct_date_str(date):
            raise ParamError('Date string should be YY-MM-DD format!')
        else:
            return date
    else:
        raise ParamError(
            'Date should be YY-MM-DD format string or datetime.date, '
            'or datetime.datetime object')


def is_legal_date_range(start, end):
    start_date = datetime.datetime.strptime(start, "%Y-%m-%d")
    end_date = datetime.datetime.strptime(end, "%Y-%m-%d")
    if (end_date - start_date).days < 0:
        return False
    else:
        return True


def check_pass_param_none(*args, **kwargs):
    if kwargs is None:
        raise ParamError("Param should not be None")

    for item in kwargs.items():
        if item[1] is None:
            raise ParamError("Param `{}` must not be None".format(item[0]))



"""
This is a client for milvus of gRPC
"""
__version__ = '0.2.3'

import grpc
import logging

from .Abstract import (
    ConnectIntf,
    IndexType,
    MetricType,
    TableSchema,
    Range,
    RowRecord,
    QueryResult,
    TopKQueryResult,
    IndexParam
)
from .utils import *
from .Status import Status
from .Exceptions import *
from ..settings import DefaultConfig as config

from ..grpc_gen import milvus_pb2_grpc, status_pb2
from ..grpc_gen import milvus_pb2 as grpc_types
from urllib.parse import urlparse

LOGGER = logging.getLogger(__name__)


class Prepare(object):

    @classmethod
    def table_name(cls, table_name):
        if not isinstance(table_name, grpc_types.TableName):
            status = status_pb2.Status(error_code=0, reason='Client')
            return grpc_types.TableName(status=status, table_name=table_name)
        else:
            return table_name

    @classmethod
    def table_schema(cls, param):
        """
        :type param: dict
        :param param: (Required)

            `example param={'table_name': 'name',
                            'dimension': 16,
                            'index_type': IndexType.FLAT,
                            'store_raw_vector': False}`

        :return: ttypes.TableSchema object
        """
        if not isinstance(param, grpc_types.TableSchema):
            if isinstance(param, dict):

                # TODO: for backward compatibility
                _param = {
                    'table_name': param['table_name'],
                    'dimension': param['dimension'],
                    'index_file_size': param['index_file_size']
                }

                temp = TableSchema(**_param)

            else:
                raise ParamError('Param type incorrect, expect {} but get {} instead '.format(
                    type(dict), type(param)
                ))

        else:
            return param

        table_name = Prepare.table_name(temp.table_name)
        return grpc_types.TableSchema(table_name=table_name,
                                      dimension=temp.dimension,
                                      index_file_size=temp.index_file_size)

    @classmethod
    def range(cls, start_date, end_date):
        """
        Parser a 'yyyy-mm-dd' like str or date/datetime object to Range object

            `Range: (start_date, end_date]`

            `start_date : '2019-05-25'`

        :param start_date: start date
        :type  start_date: str, date, datetime
        :param end_date: end date
        :type  end_date: str, date, datetime

        :return: Range object
        """
        temp = Range(start_date, end_date)

        return grpc_types.Range(start_value=temp.start_date,
                                end_value=temp.end_date)

    @classmethod
    def ranges(cls, ranges):
        """
        prepare query_ranges

        :param ranges: prepare query_ranges
        :type  ranges: [[str, str], (str,str)], iterable

            `Example: [[start, end]], ((start, end), (start, end)), or
                    [(start, end)]`

        :return: list[Range]
        """
        res = []
        for _range in ranges:
            if not isinstance(_range, grpc_types.Range):
                res.append(Prepare.range(_range[0], _range[1]))
            else:
                res.append(_range)
        return res

    @classmethod
    def insert_param(cls, table_name, vectors, ids=None):

        if ids is None:
            _param = grpc_types.InsertParam(table_name=table_name)
        else:
            if len(vectors) != len(ids):
                raise ValueError("length of vectors not match ids's")
            _param = grpc_types.InsertParam(table_name=table_name, row_id_array=ids)

        for vector in vectors:
            if is_legal_array(vector):
                _param.row_record_array.add(vector_data=vector)
            else:
                raise ParamError('Vectors should be 2-dim array!')

        # if ids is not None:
        #     if len(vectors) != len(ids):
        #         raise ParamError("Param `{0}` and `{1}` not match".format('vectors', 'ids'))
        #
        #     for _id in ids:
        #         _param.row_id_array.add(_id)

        return _param

    @classmethod
    def index(cls, index_type, nlist, metric_type):
        """

        :type index_type: IndexType
        :param index_type: index type
        :param nlist:
        :param index_file_size:

        :type  metric_type: MetricType
        :param metric_type:

        :return:
        """

        index_type = IndexType(index_type) if isinstance(index_type, int) else index_type
        if not isinstance(index_type, IndexType) or index_type == IndexType.INVALID:
            raise ParamError('Illegal index_type, should be IndexType but not IndexType.INVALID')

        metric_type = MetricType(metric_type) if isinstance(metric_type, int) else metric_type
        if not isinstance(metric_type, MetricType):
            raise ParamError('Illegal metric_type, should be MetricType')

        return grpc_types.Index(index_type=index_type, nlist=nlist, metric_type=metric_type)

    @classmethod
    def index_param(cls, table_name, index_param):

        _table_name = Prepare.table_name(table_name)

        if index_param is None:
            return grpc_types.IndexParam(table_name=_table_name)

        if not isinstance(index_param, dict):
            raise ParamError('Param type incorrect, expect {} but get {} instead '.format(
                type(dict), type(index_param)
            ))
        _index = Prepare.index(**index_param)

        return grpc_types.IndexParam(table_name=_table_name, index=_index)

    @classmethod
    def vector_ids(cls, ids):
        _status = status_pb2.Status(error_code=0, reason="OK")

        if not is_legal_array(ids):
            raise ParamError('Ids must be a list')

        return grpc_types.VectorIds(status=_status, vector_id_array=ids)

    @classmethod
    def search_param(cls, table_name, query_records, query_ranges, topk, nprobe):
        query_ranges = Prepare.ranges(query_ranges) if query_ranges else None

        search_param = grpc_types.SearchParam(
            table_name=table_name,
            query_range_array=query_ranges,
            topk=topk,
            nprobe=nprobe
        )

        for vector in query_records:
            if is_legal_array(vector):
                search_param.query_record_array.add(vector_data=vector)
            else:
                raise ParamError('Vectors should be 2-dim array!')

        return search_param

    @classmethod
    def search_vector_in_files_param(cls, table_name, query_records, query_ranges, topk, nprobe, ids):
        _search_param = Prepare.search_param(table_name, query_records, query_ranges, topk, nprobe)

        if not isinstance(ids, list):
            raise ParamError('Ids must be a list')

        return grpc_types.SearchInFilesParam(
            file_id_array=ids,
            search_param=_search_param
        )


class GrpcMilvus(ConnectIntf):
    def __init__(self):
        self._channel = None
        self._stub = None
        self._uri = None
        self.server_address = None
        self.status = None

    def __str__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items() if not key.startswith('_')]
        return '<Milvus: {}>'.format(', '.join(L))

    def set_channel(self, host=None, port=None, uri=None):

        config_uri = urlparse(config.GRPC_URI)

        _uri = urlparse(uri) if uri else config_uri

        if not host:
            if _uri.scheme == 'tcp':
                host = _uri.hostname or 'localhost'
                port = _uri.port or '19530'
            else:
                if uri:
                    raise RuntimeError(
                        'Invalid parameter uri: {}'.format(uri)
                    )
                raise RuntimeError(
                    'Invalid configuration for GRPC_URI: {}'.format(
                        config.GRPC_URI)
                )
        else:
            host = host
            port = port or '19530'

        self._uri = str(host) + ':' + str(port)
        self.server_address = self._uri
        self._channel = grpc.insecure_channel(self._uri)

    def connect(self, host=None, port=None, uri=None, timeout=5):
        """
        Connect method should be called before any operations.
        Server will be connected after connect return OK

        :type  host: str
        :type  port: str
        :type  uri: str
        :type  timeout: int
        :param host: (Optional) host of the server, default host is 127.0.0.1
        :param port: (Optional) port of the server, default port is 19530
        :param uri: (Optional) only support tcp proto now, default uri is

                `tcp://127.0.0.1:19530`

        :param timeout: (Optional) connection timeout, default timeout is 3000ms

        :return: Status, indicate if connect is successful
        :rtype: Status
        """
        if self._channel is None:
            self.set_channel(host, port, uri)

        elif self.connected():
            return Status(message="You have already connected!", code=Status.CONNECT_FAILED)

        try:
            grpc.channel_ready_future(self._channel).result(timeout=timeout)
        except grpc.FutureTimeoutError as e:
            raise NotConnectError('Fail connecting to server on {}'.format(self._uri))
        else:
            self._stub = milvus_pb2_grpc.MilvusServiceStub(self._channel)
            self.status = Status(message='Successfully connected!')
            return self.status

    def connected(self):
        """
        Check if client is connected to the server

        :return: if client is connected
        :rtype bool
        """
        if not self._stub or not self.status or not self._channel:
            return False

        try:
            grpc.channel_ready_future(self._channel).result(timeout=2)
        except grpc.FutureTimeoutError:
            return False
        else:
            return True

    def disconnect(self):
        """
        Disconnect with the server and distroy the channel

        :return: Status, indicate if disconnect is successful
        :rtype: Status
        """
        # After closeing, a exception stack trace is printed from a background thread and
        # no exception is thrown in the main thread, issue is under test and not done yet
        # checkout https://github.com/grpc/grpc/issues/18995
        # Also checkout Properly Specify Channel.close Behavior in Python:
        # https://github.com/grpc/grpc/issues/19235
        if not self.connected():
            raise DisconnectNotConnectedClientError('Please connect to the server first!')

        try:
            del self._channel
            # self._channel.close()
        except Exception as e:
            LOGGER.error(e)
            return Status(code=Status.CONNECT_FAILED, message='Disconnection failed')

        self.status = None
        self._channel = None
        self._stub = None

        return Status(message='Disconnect successfully')

    def create_table(self, param, timeout=10):
        """Create table

        :type  param: dict or TableSchema
        :param param: Provide table information to be created

                `example param={'table_name': 'name',
                                'dimension': 16,
                                'index_file_size': 1024 (optional)
                                }`

                `OR using Prepare.table_schema to create param`

        :type  timeout:
        :param timeout:

        :return: Status, indicate if operation is successful
        :rtype: Status
        """
        if not isinstance(param, dict):
            raise ParamError("param is invalid! It should be a type of dict")

        try:
            check_pass_param_none(param=param, table_name=param['table_name'], dimension=param['dimension'])
        except KeyError as e:
            raise ParamError("`param` should contain key `{}`".format(e.args[0]))

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        if 'index_file_size' not in param:
            param['index_file_size'] = 1024

        table_schema = Prepare.table_schema(param)

        try:
            status = self._stub.CreateTable.future(table_schema).result(timeout=timeout)
            if status.error_code == 0:
                return Status(message='Create table successfully!')
            elif status.error_code == status_pb2.META_FAILED:
                LOGGER.error("Table {} already exists".format(param['table_name']))
                return Status(code=status.error_code, message=status.reason)
            else:
                LOGGER.error(status)
                return Status(code=status.error_code, message=status.reason)
        except grpc.FutureTimeoutError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport timeout')
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error')

    def has_table(self, table_name, timeout=10):
        """

        This method is used to test table existence.

        :param table_name: table name is going to be tested.
        :type table_name: str

        :return:
            bool, if given table_name exists

        """
        check_pass_param_none(table_name=table_name)

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        table_name = Prepare.table_name(table_name)

        try:
            reply = self._stub.HasTable.future(table_name).result(timeout=timeout)
            if reply.status.error_code == 0:
                return reply.bool_reply
        except grpc.FutureTimeoutError as e:
            LOGGER.error(e)
            return False
        except grpc.RpcError as e:
            LOGGER.error(e)
            return False

    def delete_table(self, table_name, timeout=20):
        """
        Delete table with table_name

        :type  table_name: str
        :param table_name: Name of the table being deleted

        :return: Status, indicate if operation is successful
        :rtype: Status
        """
        check_pass_param_none(table_name=table_name)

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        table_name = Prepare.table_name(table_name)

        try:
            status = self._stub.DropTable.future(table_name).result(timeout=timeout)
            if status.error_code == 0:
                return Status(message='Delete table successfully!')
            else:
                return Status(code=status.error_code, message=status.reason)
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error')

    def create_index(self, table_name, index=None, timeout=1800):
        """
        :param table_name: table used to build index.
        :type table_name: str
        :param index: index params
        :type index: dict

            index_param can be None

            `example (default) param={'index_type': IndexType.FLAT,
                            'nlist': 16384,
                            'metric_type': MetricType.L2}`

        :return: Status, indicate if operation is successful
        """
        if index is None:
            check_pass_param_none(table_name=table_name)
        else:
            try:
                check_pass_param_none(table_name=table_name, index_type=index['index_type'], nlist=index['index_type'],
                                      metric_type=index['metric_type'])
            except KeyError as e:
                raise ParamError("Param `{}` is not allowed to be None".format(e.args[0]))

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        if index is None:
            index = {
                'index_type': IndexType.FLAT,
                'nlist': 16384,
                'metric_type': MetricType.L2
            }

        if not isinstance(index, dict):
            raise TypeError("param `index` should be a dictionary")

        index_param = Prepare.index_param(table_name, index)

        try:
            status = self._stub.CreateIndex.future(index_param).result(timeout=timeout)
            if status.error_code == 0:
                return Status(message='Build index successfully!')
            else:
                return Status(code=status.error_code, message=status.reason)
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error')

    def add_vectors(self, table_name, records, ids=None, timeout=180, *args, **kwargs):
        """
        Add vectors to table

        This function allows to pass in arguments which is type of `milvus_ob2.InsertParam`
        to avoid serializing and deserializing repeatedly, as follows:

            `obj.add_vectors(None, None, insert_param=param)`

        `obj` is a milvus object, param is an object which is type of `milvus_ob2.InsertParam`

        :param ids:

        :type  table_name: str
        :param table_name: table name been inserted

        :type  records: list[list[float]]

                `example records: [[1.2345],[1.2345]]`

                `OR using Prepare.records`

        :param records: list of vectors been inserted

        :returns:
            Status: indicate if vectors inserted successfully
            ids: list of id, after inserted every vector is given a id
        :rtype: (Status, list(int))
        """
        check_pass_param_none(table_name=table_name, records=records)

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        insert_param = kwargs.get('insert_param', None)

        if not insert_param:
            insert_param = Prepare.insert_param(table_name, records, ids)
        else:
            if not isinstance(insert_param, grpc_types.InsertParam):
                raise ParamError("The value of key 'insert_param' must be type of milvus_pb2.InsertParam")

        try:
            vector_ids = self._stub.Insert.future(insert_param).result(timeout=timeout)

            if vector_ids.status.error_code == 0:
                ids = list(vector_ids.vector_id_array)
                return Status(message='Add vectors successfully!'), ids
            else:
                return Status(code=vector_ids.status.error_code, message=vector_ids.status.reason), []
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error'), []

    def search_vectors(self, table_name, top_k, nprobe, query_records, query_ranges=None):
        """
        Query vectors in a table

        :param query_ranges: (Optional) ranges for conditional search.
            If not specified, search in the whole table
        :type  query_ranges: list[(date, date)]

                `date` supports:
                   a. date-like-str, e.g. '2019-01-01'
                   b. datetime.date object, e.g. datetime.date(2019, 1, 1)
                   c. datetime.datetime object, e.g. datetime.datetime.now()

                example query_ranges:

                `query_ranges = [('2019-05-10', '2019-05-10'),(..., ...), ...]`

        :param table_name: table name been queried
        :type  table_name: str
        :param query_records: all vectors going to be queried

                `Using Prepare.records generate query_records`

        :type  query_records: list[list[float]] or list[RowRecord]
        :param top_k: int, how many similar vectors will be searched
        :type  top_k: int
        :param nprobe: cell num of probing
        :type nprobe: int

        :returns: (Status, res)

            Status:  indicate if query is successful
            res: TopKQueryResult, return when operation is successful

        :rtype: (Status, TopKQueryResult[QueryResult])
        """

        check_pass_param_none(table_name=table_name, top_k=top_k, nprobe=nprobe)

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        if not isinstance(top_k, int) or top_k <= 0:
            raise ParamError('Param top_k should be larger than 0!')

        infos = Prepare.search_param(
            table_name, query_records, query_ranges, top_k, nprobe
        )

        results = TopKQueryResult()
        try:
            for topks in self._stub.Search(infos):
                results.append([QueryResult(id=qr.id, distance=qr.distance) for qr in topks.query_result_arrays])
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.INVALID_ARGUMENT:
                status = Status(code=Status.ILLEGAL_ARGUMENT, message=e.details())
            else:
                status = Status(code=e.code(), message='grpc transport error')

            return status, []

        return Status(message='Search vectors successfully!'), results

    def search_vectors_in_files(self, table_name, file_ids, query_records, top_k, nprobe=16, query_ranges=None):
        """
        Query vectors in a table, in specified files

        :type  nprobe: int
        :param nprobe:

        :type  table_name: str
        :param table_name: table name been queried

        :type  file_ids: list[str] or list[int]
        :param file_ids: Specified files id array

        :type  query_records: list[list[float]]
        :param query_records: all vectors going to be queried

        :param query_ranges: Optional ranges for conditional search.
            If not specified, search in the whole table


        :type  top_k: int
        :param top_k: how many similar vectors will be searched

        :returns:
            Status:  indicate if query is successful
            query_results: list[TopKQueryResult]

        :rtype: (Status, TopKQueryResult[QueryResult])
        """
        check_pass_param_none(table_name=table_name, file_ids=file_ids, query_records=query_records, top_k=top_k)

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        if not isinstance(top_k, int) or top_k <= 0:
            raise ParamError('Param top_k should be larger than 0!')

        file_ids = list(map(int_or_str, file_ids))

        infos = Prepare.search_vector_in_files_param(
            table_name, query_records, query_ranges, top_k, nprobe, file_ids
        )

        results = TopKQueryResult()
        try:
            for topks in self._stub.SearchInFiles(infos):
                if topks.status.error_code == 0:
                    results.append([QueryResult(id=qr.id, distance=qr.distance) for qr in topks.query_result_arrays])
                else:
                    return Status(code=topks.status.error_code, message=topks.status.reason), []

            return Status(message='Search vectors successfully!'), results
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.INVALID_ARGUMENT:
                status = Status(code=Status.ILLEGAL_ARGUMENT, message=e.details())
            else:
                status = Status(code=e.code(), message='grpc transport error')

            return status, []

    def describe_table(self, table_name, timeout=10):
        """
        Show table information

        :type  table_name: str
        :param table_name: which table to be shown

        :returns: (Status, table_schema)
            Status: indicate if query is successful
            table_schema: return when operation is successful
        :rtype: (Status, TableSchema)
        """
        check_pass_param_none(table_name=table_name)

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        table_name = Prepare.table_name(table_name)

        try:
            ts = self._stub.DescribeTable.future(table_name).result(timeout=timeout)

            if ts.table_name.status.error_code == 0:

                table = TableSchema(
                    table_name=ts.table_name.table_name,
                    dimension=ts.dimension,
                    index_file_size=ts.index_file_size
                )

                return Status(message='Describe table successfully!'), table
            else:
                LOGGER.error(ts.table_name.status)
                return Status(code=ts.table_name.status.error_code,
                              message=ts.table_name.status.reason), None
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error'), None

    def show_tables(self):
        """
        Show all tables in database

        :return:
            Status: indicate if this operation is successful

            tables: list of table names, return when operation
                    is successful
        :rtype:
            (Status, list[str])
        """
        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        # TODO how to status errors
        cmd = grpc_types.Command(cmd='balala')
        try:
            results = [table.table_name for table in self._stub.ShowTables(cmd)]
            return Status(message='Show tables successfully!'), results
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error'), []

    def get_table_row_count(self, table_name, timeout=10):
        """
        Get table row count

        :type  table_name: str
        :param table_name: target table name.

        :returns:
            Status: indicate if operation is successful

            res: int, table row count
        """
        check_pass_param_none(table_name=table_name)

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        table_name = Prepare.table_name(table_name)

        try:
            trc = self._stub.CountTable.future(table_name).result(timeout=timeout)
            if trc.status.error_code == 0:
                return Status(message='Get table row count successfully!'), trc.table_row_count
            else:
                return Status(code=trc.status.error_code, message=trc.status.reason), None
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error'), []

    def client_version(self):
        """
        Provide client version

        :return:
            Status: indicate if operation is successful

            str : Client version

        :rtype: (Status, str)
        """
        return __version__

    def server_version(self, timeout=10):
        """
        Provide server version

        :return:
            Status: indicate if operation is successful

            str : Server version

        :rtype: (Status, str)
        """
        return self.cmd(cmd='version', timeout=timeout)

    def server_status(self, cmd=None, timeout=10):
        """
        Provide server status. When cmd !='version', provide 'OK'

        :return:
            Status: indicate if operation is successful

            str : Server version

        :rtype: (Status, str)
        """
        return self.cmd(cmd='OK', timeout=timeout)

    def cmd(self, cmd, timeout=10):

        if not isinstance(cmd, str):
            raise TypeError("arg should be str")

        if not self.connected():
            raise NotConnectError('Please connect to the server first')

        cmd = grpc_types.Command(cmd=cmd)
        try:
            ss = self._stub.Cmd.future(cmd).result(timeout=timeout)
            if ss.status.error_code == 0:
                return Status(message='Success!'), ss.string_reply
            else:
                return Status(code=ss.status.error_code, message=ss.status.reason), None
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error'), None

    def delete_vectors_by_range(self, table_name, start_date=None, end_date=None):
        """
        Delete vectors by range

        :type  table_name: str
        :param table_name:

        :type  start_date: str
        :param start_date:

        :type  end_date:
        :param end_date:

        :return:
        """
        check_pass_param_none(table_name=table_name)

        _range = Prepare.range(start_date, end_date)
        _param = grpc_types.DeleteByRangeParam(range=_range, table_name=table_name)

        try:
            status = self._stub.DeleteByRange(_param)
            return Status(code=status.error_code, message=status.reason)
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error')

    def preload_table(self, table_name):
        """
        Load table to cache in advance

        :type table_name: str
        :param table_name: table to preload

        :returns:
            Status:  indicate if query is successful
        """
        check_pass_param_none(table_name=table_name)

        table_name = Prepare.table_name(table_name)

        try:
            status = self._stub.PreloadTable(table_name)
            return Status(code=status.error_code, message=status.reason)
        except grpc.RpcError as e:
            return Status(code=e.code(), message='grpc transport error')

    def describe_index(self, table_name, timeout=10):
        """
        Show index information of designated table

        :type table_name: str
        :param table_name: table name been queried

        :returns:
            Status:  indicate if query is successful
            IndexSchema:
            
        """
        check_pass_param_none(table_name=table_name)

        table_name = Prepare.table_name(table_name)

        try:
            index_param = self._stub.DescribeIndex.future(table_name).result(timeout=timeout)

            status = index_param.table_name.status

            if status.error_code == 0:
                index_schema = {
                    "table_name": index_param.table_name.table_name,
                    "index_type": index_param.index.index_type,
                    "nlist": index_param.index.nlist,
                    "metric_type": index_param.index.metric_type
                }

                return Status(message="Successfully"), IndexParam(**index_schema)
            else:
                return Status(code=status.error_code, message=status.reason), None
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error{}'.format(e.details())), None

    def drop_index(self, table_name, timeout=10):

        check_pass_param_none(table_name=table_name)

        table_name = Prepare.table_name(table_name)

        try:
            status = self._stub.DropIndex.future(table_name).result(timeout=timeout)
            return Status(code=status.error_code, message=status.reason)
        except grpc.RpcError as e:
            LOGGER.error(e)
            return Status(e.code(), message='grpc transport error{}'.format(e.details()))



from enum import IntEnum
import struct
from .utils import *

from milvus.client.Exceptions import (
    ParamError
)


class IndexType(IntEnum):
    INVALID = 0
    FLAT = 1
    IVFLAT = 2
    IVF_SQ8 = 3
    MIX_NSG = 4

    def __repr__(self):
        return "<{}: {}>".format(self.__class__.__name__, self._name_)

    def __str__(self):
        return self._name_


class MetricType(IntEnum):
    L2 = 1
    IP = 2

    def __repr__(self):
        return "<{}: {}>".format(self.__class__.__name__, self._name_)

    def __str__(self):
        return self._name_


class TableSchema(object):
    """
    Table Schema

    :type  table_name: str
    :param table_name: (Required) name of table

        `IndexType`: 0-invalid, 1-flat, 2-ivflat, 3-IVF_SQ8, 4-MIX_NSG

    :type  dimension: int64
    :param dimension: (Required) dimension of vector

    :type  index_file_size:
    :param index_file_size:

    """

    def __init__(self, table_name, dimension, index_file_size):

        # TODO may raise UnicodeEncodeError
        if table_name is None:
            raise ParamError('Table name can\'t be None')
        table_name = str(table_name) if not isinstance(table_name, str) else table_name
        if not legal_dimension(dimension):
            raise ParamError('Illegal dimension, effective range: (0 , 16384]')

        self.table_name = table_name
        self.dimension = dimension
        self.index_file_size = index_file_size

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))


class Range(object):
    """
    Range information

    :type  start_date: str, date or datetime

        `str should be YY-MM-DD format, e.g. "2019-07-01"`

    :param start_date: Range start date

    :type  end_date: str, date or datetime

        `str should be YY-MM-DD format, e.g. "2019-07-01"`

    :param end_date: Range end date

    """

    def __init__(self, start_date, end_date):
        start_date = parser_range_date(start_date)
        end_date = parser_range_date(end_date)
        if is_legal_date_range(start_date, end_date):
            self.start_date = start_date
            self.end_date = end_date
        else:
            raise ParamError("The start-date should be smaller"
                             " than or equal to end-date!")


class RowRecord(object):
    """
    Record inserted

    :type  vector_data: binary str
    :param vector_data: (Required) a vector

    """

    def __init__(self, vector_data):
        if isinstance(vector_data, list) and len(vector_data) > 0 \
                and isinstance(vector_data[0], float):
            self.vector_data = struct.pack(str(len(vector_data)) + 'd', *vector_data)
        else:
            raise ParamError('Illegal vector! Vector should be non-empty list of float.\n {}'
                             .format(vector_data))


class QueryResult(object):
    """
    Query result

    :type  id: int64
    :param id: id of the vector

    :type  distance: float
    :param distance: Vector similarity 0 <= distance <= 100

    """

    def __init__(self, id, distance):
        self.id = id
        self.distance = distance

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))


class TopKQueryResult(list):
    """
    TopK query results, list of QueryResult
    """

    def __repr__(self):

        if self.__len__() > 10:
            middle = ''
            for topk in self[:3]:
                middle = middle + " [ %s" % ",\n   ".join(map(str, topk[:3]))
                middle += ",\n   ..."
                middle += "\n   %s ]\n\n" % str(topk[-1])

            spaces = """        ......
        ......"""

            ahead = "[\n%s%s\n]" % (middle, spaces)
            return ahead
        else:
            return "[\n%s\n]" % ",\n".join(map(str, self))


class IndexParam(object):
    """
    Index Param

    :type  table_name: str
    :param table_name: (Required) name of table

    :type  index_type: IndexType
    :param index_type: (Required) index type, default = IndexType.INVALID

        `IndexType`: 0-invalid, 1-flat, 2-ivflat, 3-IVF_SQ8, 4-MIX_NSG

    :type  nlist: int64
    :param nlist: (Required) num of cell

    :type  metric_type: int32
    :param metric_type: ???
    """

    def __init__(self, table_name, index_type, nlist, metric_type):

        if table_name is None:
            raise ParamError('Table name can\'t be None')
        table_name = str(table_name) if not isinstance(table_name, str) else table_name

        if isinstance(index_type, int):
            index_type = IndexType(index_type)
        if not isinstance(index_type, IndexType) or index_type == IndexType.INVALID:
            raise ParamError('Illegal index_type, should be IndexType but not IndexType.INVALID')

        if isinstance(metric_type, int):
            metric_type = MetricType(metric_type)
        if not isinstance(metric_type, MetricType):
            raise ParamError('Illegal metric_type, should be MetricType')

        self._table_name = table_name
        self._index_type = index_type
        self._nlist = nlist
        self._metric_type = metric_type

    def __str__(self):
        L = ['%s=%r' % (key.lstrip('_'), value)
             for key, value in self.__dict__.items()]
        return '(%s)' % (', '.join(L))

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))


def _abstract():
    raise NotImplementedError('You need to override this function')


class ConnectIntf(object):
    """SDK client abstract class

    Connection is a abstract class

    """

    def connect(self, host=None, port=None, uri=None):
        """
        Connect method should be called before any operations
        Server will be connected after connect return OK
        Should be implemented

        :type  host: str
        :param host: host

        :type  port: str
        :param port: port

        :type  uri: str
        :param uri: (Optional) uri

        :return: Status,  indicate if connect is successful
        """
        _abstract()

    def connected(self):
        """
        connected, connection status
        Should be implemented

        :return: Status,  indicate if connect is successful
        """
        _abstract()

    def disconnect(self):
        """
        Disconnect, server will be disconnected after disconnect return SUCCESS
        Should be implemented

        :return: Status,  indicate if connect is successful
        """
        _abstract()

    def create_table(self, param):
        """
        Create table
        Should be implemented

        :type  param: TableSchema
        :param param: provide table information to be created

        :return: Status, indicate if connect is successful
        """
        _abstract()

    def has_table(self, table_name):
        """

        This method is used to test table existence.
        Should be implemented

        :param table_name: table name is going to be tested.
        :type table_name: str

        :return:
            has_table: bool, if given table_name exists

        """
        _abstract()

    def build_index(self, table_name):
        """
        Build index by table name

        This method is used to build index by table in sync mode.

        :param table_name: table is going to be built index.
        :type  table_name: str

        :return: Status, indicate if operation is successful
        """
        _abstract()

    def delete_table(self, table_name):
        """
        Delete table
        Should be implemented

        :type  table_name: str
        :param table_name: table_name of the deleting table

        :return: Status, indicate if connect is successful
        """
        _abstract()

    def add_vectors(self, table_name, records):
        """
        Add vectors to table
        Should be implemented

        :type  table_name: str
        :param table_name: table name been inserted

        :type  records: list[RowRecord]
        :param records: list of vectors been inserted

        :returns:
            Status : indicate if vectors inserted successfully
            ids :list of id, after inserted every vector is given a id
        """
        _abstract()

    def search_vectors(self, table_name, query_records, query_ranges, top_k):
        """
        Query vectors in a table
        Should be implemented

        :type  table_name: str
        :param table_name: table name been queried

        :type  query_records: list[RowRecord]
        :param query_records: all vectors going to be queried

        :type  query_ranges: list[Range]
        :param query_ranges: Optional ranges for conditional search.
            If not specified, search whole table

        :type  top_k: int
        :param top_k: how many similar vectors will be searched

        :returns:
            Status:  indicate if query is successful
            query_results: list[TopKQueryResult]
        """
        _abstract()

    def search_vectors_in_files(self, table_name, file_ids, query_records, query_ranges, top_k):
        """
        Query vectors in a table, query vector in specified files
        Should be implemented

        :type  table_name: str
        :param table_name: table name been queried

        :type  file_ids: list[str]
        :param file_ids: Specified files id array

        :type  query_records: list[RowRecord]
        :param query_records: all vectors going to be queried

        :type  query_ranges: list[Range]
        :param query_ranges: Optional ranges for conditional search.
            If not specified, search whole table

        :type  top_k: int
        :param top_k: how many similar vectors will be searched

        :returns:
            Status:  indicate if query is successful
            query_results: list[TopKQueryResult]
        """
        _abstract()

    def describe_table(self, table_name):
        """
        Show table information
        Should be implemented

        :type  table_name: str
        :param table_name: which table to be shown

        :returns:
            Status: indicate if query is successful
            table_schema: TableSchema, given when operation is successful
        """
        _abstract()

    def get_table_row_count(self, table_name):
        """
        Get table row count
        Should be implemented

        :type  table_name, str
        :param table_name, target table name.

        :returns:
            Status: indicate if operation is successful
            count: int, table row count
        """
        _abstract()

    def show_tables(self):
        """
        Show all tables in database
        should be implemented

        :return:
            Status: indicate if this operation is successful
            tables: list[str], list of table names
        """
        _abstract()

    def client_version(self):
        """
        Provide client version
        should be implemented

        :return:
            Status: indicate if operation is successful

            str : Client version

        :rtype: (Status, str)
        """
        _abstract()

    def server_version(self):
        """
        Provide server version
        should be implemented

        :return:
            Status: indicate if operation is successful

            str : Server version

        :rtype: (Status, str)
        """
        _abstract()

    def server_status(self, cmd):
        """
        Provide server status. When cmd !='version', provide 'OK'
        should be implemented

        :return:
            Status: indicate if operation is successful

            str : Server version

        :rtype: (Status, str)
        """
        _abstract()

    def delete_vectors_by_range(self, start_time, end_time):
        """

        :param start_time:
        :param end_time:
        :return:
        """

        _abstract()

    def preload_table(self, table_name):
        """
        load table to cache in advance
        should be implemented

        :param table_name: target table name.
        :type table_name: str

        :return:
        """

        _abstract()

    def describe_index(self, table_name):
        """
        Show index information
        should be implemented

        :param table_name: target table name.
        :type table_name: str

        :return:
        """

        _abstract()

    def drop_index(self, table_name):
        """
        Show index information
        should be implemented

        :param table_name: target table name.
        :type table_name: str

        :return:
        """

        _abstract()



import os
import sys
import pathlib


root_path = pathlib.Path(os.path.abspath(__file__)).parent
sys.path.append(str(root_path))

from milvus.grpc_gen import status_pb2
from milvus.grpc_gen import status_pb2_grpc
from milvus.grpc_gen import milvus_pb2
from milvus.grpc_gen import milvus_pb2_grpc


__all__ = ['status_pb2', 'status_pb2_grpc', 'milvus_pb2', 'milvus_pb2_grpc']



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: milvus.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from . import status_pb2 as status__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='milvus.proto',
    package='milvus.grpc',
    syntax='proto3',
    serialized_options=None,
    serialized_pb=_b(
        '\n\x0cmilvus.proto\x12\x0bmilvus.grpc\x1a\x0cstatus.proto\"D\n\tTableName\x12#\n\x06status\x18\x01 \x01(\x0b\x32\x13.milvus.grpc.Status\x12\x12\n\ntable_name\x18\x02 \x01(\t\"e\n\x0bTableSchema\x12*\n\ntable_name\x18\x01 \x01(\x0b\x32\x16.milvus.grpc.TableName\x12\x11\n\tdimension\x18\x02 \x01(\x03\x12\x17\n\x0findex_file_size\x18\x03 \x01(\x03\"/\n\x05Range\x12\x13\n\x0bstart_value\x18\x01 \x01(\t\x12\x11\n\tend_value\x18\x02 \x01(\t\" \n\tRowRecord\x12\x13\n\x0bvector_data\x18\x01 \x03(\x02\"i\n\x0bInsertParam\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x30\n\x10row_record_array\x18\x02 \x03(\x0b\x32\x16.milvus.grpc.RowRecord\x12\x14\n\x0crow_id_array\x18\x03 \x03(\x03\"I\n\tVectorIds\x12#\n\x06status\x18\x01 \x01(\x0b\x32\x13.milvus.grpc.Status\x12\x17\n\x0fvector_id_array\x18\x02 \x03(\x03\"\xa2\x01\n\x0bSearchParam\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x32\n\x12query_record_array\x18\x02 \x03(\x0b\x32\x16.milvus.grpc.RowRecord\x12-\n\x11query_range_array\x18\x03 \x03(\x0b\x32\x12.milvus.grpc.Range\x12\x0c\n\x04topk\x18\x04 \x01(\x03\x12\x0e\n\x06nprobe\x18\x05 \x01(\x03\"[\n\x12SearchInFilesParam\x12\x15\n\rfile_id_array\x18\x01 \x03(\t\x12.\n\x0csearch_param\x18\x02 \x01(\x0b\x32\x18.milvus.grpc.SearchParam\"+\n\x0bQueryResult\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x10\n\x08\x64istance\x18\x02 \x01(\x01\"m\n\x0fTopKQueryResult\x12#\n\x06status\x18\x01 \x01(\x0b\x32\x13.milvus.grpc.Status\x12\x35\n\x13query_result_arrays\x18\x02 \x03(\x0b\x32\x18.milvus.grpc.QueryResult\"H\n\x0bStringReply\x12#\n\x06status\x18\x01 \x01(\x0b\x32\x13.milvus.grpc.Status\x12\x14\n\x0cstring_reply\x18\x02 \x01(\t\"D\n\tBoolReply\x12#\n\x06status\x18\x01 \x01(\x0b\x32\x13.milvus.grpc.Status\x12\x12\n\nbool_reply\x18\x02 \x01(\x08\"M\n\rTableRowCount\x12#\n\x06status\x18\x01 \x01(\x0b\x32\x13.milvus.grpc.Status\x12\x17\n\x0ftable_row_count\x18\x02 \x01(\x03\"\x16\n\x07\x43ommand\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"?\n\x05Index\x12\x12\n\nindex_type\x18\x01 \x01(\x05\x12\r\n\x05nlist\x18\x02 \x01(\x05\x12\x13\n\x0bmetric_type\x18\x03 \x01(\x05\"[\n\nIndexParam\x12*\n\ntable_name\x18\x01 \x01(\x0b\x32\x16.milvus.grpc.TableName\x12!\n\x05index\x18\x02 \x01(\x0b\x32\x12.milvus.grpc.Index\"K\n\x12\x44\x65leteByRangeParam\x12!\n\x05range\x18\x01 \x01(\x0b\x32\x12.milvus.grpc.Range\x12\x12\n\ntable_name\x18\x02 \x01(\t2\xea\x07\n\rMilvusService\x12>\n\x0b\x43reateTable\x12\x18.milvus.grpc.TableSchema\x1a\x13.milvus.grpc.Status\"\x00\x12<\n\x08HasTable\x12\x16.milvus.grpc.TableName\x1a\x16.milvus.grpc.BoolReply\"\x00\x12:\n\tDropTable\x12\x16.milvus.grpc.TableName\x1a\x13.milvus.grpc.Status\"\x00\x12=\n\x0b\x43reateIndex\x12\x17.milvus.grpc.IndexParam\x1a\x13.milvus.grpc.Status\"\x00\x12<\n\x06Insert\x12\x18.milvus.grpc.InsertParam\x1a\x16.milvus.grpc.VectorIds\"\x00\x12\x44\n\x06Search\x12\x18.milvus.grpc.SearchParam\x1a\x1c.milvus.grpc.TopKQueryResult\"\x00\x30\x01\x12R\n\rSearchInFiles\x12\x1f.milvus.grpc.SearchInFilesParam\x1a\x1c.milvus.grpc.TopKQueryResult\"\x00\x30\x01\x12\x43\n\rDescribeTable\x12\x16.milvus.grpc.TableName\x1a\x18.milvus.grpc.TableSchema\"\x00\x12\x42\n\nCountTable\x12\x16.milvus.grpc.TableName\x1a\x1a.milvus.grpc.TableRowCount\"\x00\x12>\n\nShowTables\x12\x14.milvus.grpc.Command\x1a\x16.milvus.grpc.TableName\"\x00\x30\x01\x12\x37\n\x03\x43md\x12\x14.milvus.grpc.Command\x1a\x18.milvus.grpc.StringReply\"\x00\x12G\n\rDeleteByRange\x12\x1f.milvus.grpc.DeleteByRangeParam\x1a\x13.milvus.grpc.Status\"\x00\x12=\n\x0cPreloadTable\x12\x16.milvus.grpc.TableName\x1a\x13.milvus.grpc.Status\"\x00\x12\x42\n\rDescribeIndex\x12\x16.milvus.grpc.TableName\x1a\x17.milvus.grpc.IndexParam\"\x00\x12:\n\tDropIndex\x12\x16.milvus.grpc.TableName\x1a\x13.milvus.grpc.Status\"\x00\x62\x06proto3')
    ,
    dependencies=[status__pb2.DESCRIPTOR, ])

_TABLENAME = _descriptor.Descriptor(
    name='TableName',
    full_name='milvus.grpc.TableName',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='status', full_name='milvus.grpc.TableName.status', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='table_name', full_name='milvus.grpc.TableName.table_name', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=43,
    serialized_end=111,
)

_TABLESCHEMA = _descriptor.Descriptor(
    name='TableSchema',
    full_name='milvus.grpc.TableSchema',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='table_name', full_name='milvus.grpc.TableSchema.table_name', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='dimension', full_name='milvus.grpc.TableSchema.dimension', index=1,
            number=2, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='index_file_size', full_name='milvus.grpc.TableSchema.index_file_size', index=2,
            number=3, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=113,
    serialized_end=214,
)

_RANGE = _descriptor.Descriptor(
    name='Range',
    full_name='milvus.grpc.Range',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='start_value', full_name='milvus.grpc.Range.start_value', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='end_value', full_name='milvus.grpc.Range.end_value', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=216,
    serialized_end=263,
)

_ROWRECORD = _descriptor.Descriptor(
    name='RowRecord',
    full_name='milvus.grpc.RowRecord',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='vector_data', full_name='milvus.grpc.RowRecord.vector_data', index=0,
            number=1, type=2, cpp_type=6, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=265,
    serialized_end=297,
)

_INSERTPARAM = _descriptor.Descriptor(
    name='InsertParam',
    full_name='milvus.grpc.InsertParam',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='table_name', full_name='milvus.grpc.InsertParam.table_name', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='row_record_array', full_name='milvus.grpc.InsertParam.row_record_array', index=1,
            number=2, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='row_id_array', full_name='milvus.grpc.InsertParam.row_id_array', index=2,
            number=3, type=3, cpp_type=2, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=299,
    serialized_end=404,
)

_VECTORIDS = _descriptor.Descriptor(
    name='VectorIds',
    full_name='milvus.grpc.VectorIds',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='status', full_name='milvus.grpc.VectorIds.status', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='vector_id_array', full_name='milvus.grpc.VectorIds.vector_id_array', index=1,
            number=2, type=3, cpp_type=2, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=406,
    serialized_end=479,
)

_SEARCHPARAM = _descriptor.Descriptor(
    name='SearchParam',
    full_name='milvus.grpc.SearchParam',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='table_name', full_name='milvus.grpc.SearchParam.table_name', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='query_record_array', full_name='milvus.grpc.SearchParam.query_record_array', index=1,
            number=2, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='query_range_array', full_name='milvus.grpc.SearchParam.query_range_array', index=2,
            number=3, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='topk', full_name='milvus.grpc.SearchParam.topk', index=3,
            number=4, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='nprobe', full_name='milvus.grpc.SearchParam.nprobe', index=4,
            number=5, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=482,
    serialized_end=644,
)

_SEARCHINFILESPARAM = _descriptor.Descriptor(
    name='SearchInFilesParam',
    full_name='milvus.grpc.SearchInFilesParam',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='file_id_array', full_name='milvus.grpc.SearchInFilesParam.file_id_array', index=0,
            number=1, type=9, cpp_type=9, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='search_param', full_name='milvus.grpc.SearchInFilesParam.search_param', index=1,
            number=2, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=646,
    serialized_end=737,
)

_QUERYRESULT = _descriptor.Descriptor(
    name='QueryResult',
    full_name='milvus.grpc.QueryResult',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='id', full_name='milvus.grpc.QueryResult.id', index=0,
            number=1, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='distance', full_name='milvus.grpc.QueryResult.distance', index=1,
            number=2, type=1, cpp_type=5, label=1,
            has_default_value=False, default_value=float(0),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=739,
    serialized_end=782,
)

_TOPKQUERYRESULT = _descriptor.Descriptor(
    name='TopKQueryResult',
    full_name='milvus.grpc.TopKQueryResult',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='status', full_name='milvus.grpc.TopKQueryResult.status', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='query_result_arrays', full_name='milvus.grpc.TopKQueryResult.query_result_arrays', index=1,
            number=2, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=784,
    serialized_end=893,
)

_STRINGREPLY = _descriptor.Descriptor(
    name='StringReply',
    full_name='milvus.grpc.StringReply',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='status', full_name='milvus.grpc.StringReply.status', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='string_reply', full_name='milvus.grpc.StringReply.string_reply', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=895,
    serialized_end=967,
)

_BOOLREPLY = _descriptor.Descriptor(
    name='BoolReply',
    full_name='milvus.grpc.BoolReply',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='status', full_name='milvus.grpc.BoolReply.status', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='bool_reply', full_name='milvus.grpc.BoolReply.bool_reply', index=1,
            number=2, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=969,
    serialized_end=1037,
)

_TABLEROWCOUNT = _descriptor.Descriptor(
    name='TableRowCount',
    full_name='milvus.grpc.TableRowCount',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='status', full_name='milvus.grpc.TableRowCount.status', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='table_row_count', full_name='milvus.grpc.TableRowCount.table_row_count', index=1,
            number=2, type=3, cpp_type=2, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=1039,
    serialized_end=1116,
)

_COMMAND = _descriptor.Descriptor(
    name='Command',
    full_name='milvus.grpc.Command',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='cmd', full_name='milvus.grpc.Command.cmd', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=1118,
    serialized_end=1140,
)

_INDEX = _descriptor.Descriptor(
    name='Index',
    full_name='milvus.grpc.Index',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='index_type', full_name='milvus.grpc.Index.index_type', index=0,
            number=1, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='nlist', full_name='milvus.grpc.Index.nlist', index=1,
            number=2, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='metric_type', full_name='milvus.grpc.Index.metric_type', index=2,
            number=3, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=1142,
    serialized_end=1205,
)

_INDEXPARAM = _descriptor.Descriptor(
    name='IndexParam',
    full_name='milvus.grpc.IndexParam',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='table_name', full_name='milvus.grpc.IndexParam.table_name', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='index', full_name='milvus.grpc.IndexParam.index', index=1,
            number=2, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=1207,
    serialized_end=1298,
)

_DELETEBYRANGEPARAM = _descriptor.Descriptor(
    name='DeleteByRangeParam',
    full_name='milvus.grpc.DeleteByRangeParam',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='range', full_name='milvus.grpc.DeleteByRangeParam.range', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='table_name', full_name='milvus.grpc.DeleteByRangeParam.table_name', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=1300,
    serialized_end=1375,
)

_TABLENAME.fields_by_name['status'].message_type = status__pb2._STATUS
_TABLESCHEMA.fields_by_name['table_name'].message_type = _TABLENAME
_INSERTPARAM.fields_by_name['row_record_array'].message_type = _ROWRECORD
_VECTORIDS.fields_by_name['status'].message_type = status__pb2._STATUS
_SEARCHPARAM.fields_by_name['query_record_array'].message_type = _ROWRECORD
_SEARCHPARAM.fields_by_name['query_range_array'].message_type = _RANGE
_SEARCHINFILESPARAM.fields_by_name['search_param'].message_type = _SEARCHPARAM
_TOPKQUERYRESULT.fields_by_name['status'].message_type = status__pb2._STATUS
_TOPKQUERYRESULT.fields_by_name['query_result_arrays'].message_type = _QUERYRESULT
_STRINGREPLY.fields_by_name['status'].message_type = status__pb2._STATUS
_BOOLREPLY.fields_by_name['status'].message_type = status__pb2._STATUS
_TABLEROWCOUNT.fields_by_name['status'].message_type = status__pb2._STATUS
_INDEXPARAM.fields_by_name['table_name'].message_type = _TABLENAME
_INDEXPARAM.fields_by_name['index'].message_type = _INDEX
_DELETEBYRANGEPARAM.fields_by_name['range'].message_type = _RANGE
DESCRIPTOR.message_types_by_name['TableName'] = _TABLENAME
DESCRIPTOR.message_types_by_name['TableSchema'] = _TABLESCHEMA
DESCRIPTOR.message_types_by_name['Range'] = _RANGE
DESCRIPTOR.message_types_by_name['RowRecord'] = _ROWRECORD
DESCRIPTOR.message_types_by_name['InsertParam'] = _INSERTPARAM
DESCRIPTOR.message_types_by_name['VectorIds'] = _VECTORIDS
DESCRIPTOR.message_types_by_name['SearchParam'] = _SEARCHPARAM
DESCRIPTOR.message_types_by_name['SearchInFilesParam'] = _SEARCHINFILESPARAM
DESCRIPTOR.message_types_by_name['QueryResult'] = _QUERYRESULT
DESCRIPTOR.message_types_by_name['TopKQueryResult'] = _TOPKQUERYRESULT
DESCRIPTOR.message_types_by_name['StringReply'] = _STRINGREPLY
DESCRIPTOR.message_types_by_name['BoolReply'] = _BOOLREPLY
DESCRIPTOR.message_types_by_name['TableRowCount'] = _TABLEROWCOUNT
DESCRIPTOR.message_types_by_name['Command'] = _COMMAND
DESCRIPTOR.message_types_by_name['Index'] = _INDEX
DESCRIPTOR.message_types_by_name['IndexParam'] = _INDEXPARAM
DESCRIPTOR.message_types_by_name['DeleteByRangeParam'] = _DELETEBYRANGEPARAM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TableName = _reflection.GeneratedProtocolMessageType('TableName', (_message.Message,), {
    'DESCRIPTOR': _TABLENAME,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.TableName)
})
_sym_db.RegisterMessage(TableName)

TableSchema = _reflection.GeneratedProtocolMessageType('TableSchema', (_message.Message,), {
    'DESCRIPTOR': _TABLESCHEMA,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.TableSchema)
})
_sym_db.RegisterMessage(TableSchema)

Range = _reflection.GeneratedProtocolMessageType('Range', (_message.Message,), {
    'DESCRIPTOR': _RANGE,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.Range)
})
_sym_db.RegisterMessage(Range)

RowRecord = _reflection.GeneratedProtocolMessageType('RowRecord', (_message.Message,), {
    'DESCRIPTOR': _ROWRECORD,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.RowRecord)
})
_sym_db.RegisterMessage(RowRecord)

InsertParam = _reflection.GeneratedProtocolMessageType('InsertParam', (_message.Message,), {
    'DESCRIPTOR': _INSERTPARAM,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.InsertParam)
})
_sym_db.RegisterMessage(InsertParam)

VectorIds = _reflection.GeneratedProtocolMessageType('VectorIds', (_message.Message,), {
    'DESCRIPTOR': _VECTORIDS,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.VectorIds)
})
_sym_db.RegisterMessage(VectorIds)

SearchParam = _reflection.GeneratedProtocolMessageType('SearchParam', (_message.Message,), {
    'DESCRIPTOR': _SEARCHPARAM,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.SearchParam)
})
_sym_db.RegisterMessage(SearchParam)

SearchInFilesParam = _reflection.GeneratedProtocolMessageType('SearchInFilesParam', (_message.Message,), {
    'DESCRIPTOR': _SEARCHINFILESPARAM,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.SearchInFilesParam)
})
_sym_db.RegisterMessage(SearchInFilesParam)

QueryResult = _reflection.GeneratedProtocolMessageType('QueryResult', (_message.Message,), {
    'DESCRIPTOR': _QUERYRESULT,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.QueryResult)
})
_sym_db.RegisterMessage(QueryResult)

TopKQueryResult = _reflection.GeneratedProtocolMessageType('TopKQueryResult', (_message.Message,), {
    'DESCRIPTOR': _TOPKQUERYRESULT,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.TopKQueryResult)
})
_sym_db.RegisterMessage(TopKQueryResult)

StringReply = _reflection.GeneratedProtocolMessageType('StringReply', (_message.Message,), {
    'DESCRIPTOR': _STRINGREPLY,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.StringReply)
})
_sym_db.RegisterMessage(StringReply)

BoolReply = _reflection.GeneratedProtocolMessageType('BoolReply', (_message.Message,), {
    'DESCRIPTOR': _BOOLREPLY,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.BoolReply)
})
_sym_db.RegisterMessage(BoolReply)

TableRowCount = _reflection.GeneratedProtocolMessageType('TableRowCount', (_message.Message,), {
    'DESCRIPTOR': _TABLEROWCOUNT,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.TableRowCount)
})
_sym_db.RegisterMessage(TableRowCount)

Command = _reflection.GeneratedProtocolMessageType('Command', (_message.Message,), {
    'DESCRIPTOR': _COMMAND,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.Command)
})
_sym_db.RegisterMessage(Command)

Index = _reflection.GeneratedProtocolMessageType('Index', (_message.Message,), {
    'DESCRIPTOR': _INDEX,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.Index)
})
_sym_db.RegisterMessage(Index)

IndexParam = _reflection.GeneratedProtocolMessageType('IndexParam', (_message.Message,), {
    'DESCRIPTOR': _INDEXPARAM,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.IndexParam)
})
_sym_db.RegisterMessage(IndexParam)

DeleteByRangeParam = _reflection.GeneratedProtocolMessageType('DeleteByRangeParam', (_message.Message,), {
    'DESCRIPTOR': _DELETEBYRANGEPARAM,
    '__module__': 'milvus_pb2'
    # @@protoc_insertion_point(class_scope:milvus.grpc.DeleteByRangeParam)
})
_sym_db.RegisterMessage(DeleteByRangeParam)

_MILVUSSERVICE = _descriptor.ServiceDescriptor(
    name='MilvusService',
    full_name='milvus.grpc.MilvusService',
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    serialized_start=1378,
    serialized_end=2380,
    methods=[
        _descriptor.MethodDescriptor(
            name='CreateTable',
            full_name='milvus.grpc.MilvusService.CreateTable',
            index=0,
            containing_service=None,
            input_type=_TABLESCHEMA,
            output_type=status__pb2._STATUS,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='HasTable',
            full_name='milvus.grpc.MilvusService.HasTable',
            index=1,
            containing_service=None,
            input_type=_TABLENAME,
            output_type=_BOOLREPLY,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='DropTable',
            full_name='milvus.grpc.MilvusService.DropTable',
            index=2,
            containing_service=None,
            input_type=_TABLENAME,
            output_type=status__pb2._STATUS,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='CreateIndex',
            full_name='milvus.grpc.MilvusService.CreateIndex',
            index=3,
            containing_service=None,
            input_type=_INDEXPARAM,
            output_type=status__pb2._STATUS,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='Insert',
            full_name='milvus.grpc.MilvusService.Insert',
            index=4,
            containing_service=None,
            input_type=_INSERTPARAM,
            output_type=_VECTORIDS,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='Search',
            full_name='milvus.grpc.MilvusService.Search',
            index=5,
            containing_service=None,
            input_type=_SEARCHPARAM,
            output_type=_TOPKQUERYRESULT,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='SearchInFiles',
            full_name='milvus.grpc.MilvusService.SearchInFiles',
            index=6,
            containing_service=None,
            input_type=_SEARCHINFILESPARAM,
            output_type=_TOPKQUERYRESULT,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='DescribeTable',
            full_name='milvus.grpc.MilvusService.DescribeTable',
            index=7,
            containing_service=None,
            input_type=_TABLENAME,
            output_type=_TABLESCHEMA,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='CountTable',
            full_name='milvus.grpc.MilvusService.CountTable',
            index=8,
            containing_service=None,
            input_type=_TABLENAME,
            output_type=_TABLEROWCOUNT,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='ShowTables',
            full_name='milvus.grpc.MilvusService.ShowTables',
            index=9,
            containing_service=None,
            input_type=_COMMAND,
            output_type=_TABLENAME,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='Cmd',
            full_name='milvus.grpc.MilvusService.Cmd',
            index=10,
            containing_service=None,
            input_type=_COMMAND,
            output_type=_STRINGREPLY,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='DeleteByRange',
            full_name='milvus.grpc.MilvusService.DeleteByRange',
            index=11,
            containing_service=None,
            input_type=_DELETEBYRANGEPARAM,
            output_type=status__pb2._STATUS,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='PreloadTable',
            full_name='milvus.grpc.MilvusService.PreloadTable',
            index=12,
            containing_service=None,
            input_type=_TABLENAME,
            output_type=status__pb2._STATUS,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='DescribeIndex',
            full_name='milvus.grpc.MilvusService.DescribeIndex',
            index=13,
            containing_service=None,
            input_type=_TABLENAME,
            output_type=_INDEXPARAM,
            serialized_options=None,
        ),
        _descriptor.MethodDescriptor(
            name='DropIndex',
            full_name='milvus.grpc.MilvusService.DropIndex',
            index=14,
            containing_service=None,
            input_type=_TABLENAME,
            output_type=status__pb2._STATUS,
            serialized_options=None,
        ),
    ])
_sym_db.RegisterServiceDescriptor(_MILVUSSERVICE)

DESCRIPTOR.services_by_name['MilvusService'] = _MILVUSSERVICE

# @@protoc_insertion_point(module_scope)



from .client.GrpcClient import GrpcMilvus as Milvus
from .client.GrpcClient import Prepare
from .client.Abstract import IndexType, MetricType
from .client.Status import Status
from .client import Exceptions as milvusError

__all__ = ['Milvus', 'Prepare', 'Status', 'IndexType', 'MetricType', 'milvusError', '__version__']

__version__ = '0.2.3'



class Status(object):
    """
    :attribute code: int (optional) default as ok

    :attribute message: str (optional) current status message
    """
    SUCCESS = 0
    UNEXPECTED_ERROR = 1
    CONNECT_FAILED = 2
    PERMISSION_DENIED = 3
    TABLE_NOT_EXISTS = 4
    ILLEGAL_ARGUMENT = 5
    ILLEGAL_RANGE = 6
    ILLEGAL_DIMENSION = 7
    ILLEGAL_INDEX_TYPE = 8
    ILLEGAL_TABLE_NAME = 9
    ILLEGAL_TOPK = 10
    ILLEGAL_ROWRECORD = 11
    ILLEGAL_VECTOR_ID = 12
    ILLEGAL_SEARCH_RESULT = 13
    FILE_NOT_FOUND = 14
    META_FAILED = 15
    CACHE_FAILED = 16
    CANNOT_CREATE_FOLDER = 17
    CANNOT_CREATE_FILE = 18
    CANNOT_DELETE_FOLDER = 19
    CANNOT_DELETE_FILE = 20

    def __init__(self, code=SUCCESS, message=None):
        self.code = code
        self.message = message

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        """Make Status comparable with self by code"""
        if isinstance(other, int):
            return self.code == other
        else:
            return isinstance(other, self.__class__) and self.code == other.code

    def __ne__(self, other):
        return not (self == other)

    def OK(self):
        return self.code == Status.SUCCESS




class ParamError(ValueError):
    """
    Param of interface is illegal
    """
    pass


class ConnectError(ValueError):
    """
    Connect server failed
    """
    pass


class NotConnectError(ConnectError):
    pass


class RepeatingConnectError(ConnectError):
    pass


class DisconnectNotConnectedClientError(ConnectError):
    pass



import pathlib
import setuptools

HERE = pathlib.Path(__file__).parent

README = (HERE / 'README.md').read_text()

setuptools.setup(
    name="pymilvus-test",
    version="0.2.3",

    description="Python Sdk for Milvus; Alpha version",
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/BossZou/pymilvus',
    license="Apache-2.0",
    packages=["milvus.client", 'milvus.grpc_gen', 'milvus'],
    include_package_data=True,
    install_requires=["grpcio==1.22.0", "grpcio-tools==1.22.0"],
    classifiers=[
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],

    python_requires='>=3.4'
)
